CREATE USER taboodemo  WITH PASSWORD 'hk987JKBgui';

GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE categories TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE comments TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE stories TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE users TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE users_id_seq TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE categories_id_seq TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE contributors TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE articles TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE articlecats TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE articleusers TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE articleusers_id_seq TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE articleuserroles_id_seq TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE articlecats_id_seq TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE articles_id_seq TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE languages_id_seq TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE mediatypes_id_seq TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE articleuserroles TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE languages TO taboodemo;
GRANT SELECT, INSERT, UPDATE, DELETE ON TABLE mediatypes TO taboodemo;
